@@ -28,6 +28,3 @@ class AutonML(object):
         if proc.returncode != 0:
             print(output)
             raise RuntimeError(error.decode())
-
-        # Remove extra variables and 
-        garbage_collect()
