@@ -6,6 +6,7 @@
 # Copyright (c) 2021 Carnegie Mellon University
 # This code is subject to the license terms contained in the code repo.
 
+from ctypes import ArgumentError
 import logging
 from multiprocessing import set_start_method
 set_start_method("spawn", force=True)
@@ -13,6 +14,8 @@ from concurrent import futures
 import time
 import sys
 import os
+import json
+from pathlib import Path
 import shutil
 import warnings
 import subprocess
@@ -38,9 +41,38 @@ def garbage_collect():
     if os.path.exists(os.path.join(os.getcwd(), "nbeats_weights")):
         shutil.rmtree(os.path.join(os.getcwd(), "nbeats_weights"))
 
+def prerun_error_check():
+    if len(sys.argv) < 6 or len(sys.argv) > 7:
+        arg_err_str = "Incorrect number of arguments provided. Required arguments: \n\t-Input Directory \n\t-Output Directory \n\t-Timeout(in minutes) \n\t-Number of CPU cores \n\t-D3M Problem Path (please refer to AutonML's documentation for more details \n\n Optional arguments \n\t-D3M Static directory: stores weight files\n"
+        raise ArgumentError(arg_err_str)
+
+    if not os.path.exists(sys.argv[1]):
+        raise FileNotFoundError("Input directory does not exist")
+
+    if not os.path.exists(sys.argv[5]):
+        raise FileNotFoundError("Problem Path directory does not exist")
+
+    if len(sys.argv) == 7:
+        if not os.path.exists(sys.argv[6]):
+            raise FileNotFoundError("Static directory does not exist")
+
 def main_run():
     '''Main API to invoke complete AutoML pipeline'''
 
+    if not os.path.exists(sys.argv[2]):
+        logging.warning(f"Output directory does not exist, creating Output directory at path: {sys.argv[2]}")
+
+        output_path = Path(sys.argv[2])
+
+        if not output_path.parent.exists():
+            # TODO (vedant) : should parent directories be created recursively too?
+            raise FileNotFoundError(f"Parent directory {output_path.parent()} for provided output path: {sys.argv[2]} does not exist. Please recheck that the provided parent path is correct")
+        else:
+            os.makedir(sys.argv[2])
+
+    # Perform a preliminary error check to ensure number of arguments and directories supplied are correct
+    prerun_error_check()
+
     # Remove all files in output directory
     output_dir = sys.argv[2]
     for search_dir in os.listdir(output_dir):
@@ -54,6 +86,21 @@ def main_run():
     # TODO (vedant) : error checks here
     fit_and_evaluate(output_dir)
 
+    # Save environment variables in a file
+    for search_dir in os.listdir(sys.argv[2]):
+        search_dir_path = os.path.join(sys.argv[2], search_dir)
+        filepath = os.path.join(search_dir_path, 'environment_variables.json')
+        
+        env_dict = {}
+        env_dict['D3MINPUTDIR'] = os.environ['D3MINPUTDIR']
+        env_dict['D3MSTATICDIR'] = os.environ['D3MSTATICDIR']
+        env_dict['D3MOUTPUTDIR'] = os.environ['D3MOUTPUTDIR']
+        env_dict['D3MTIMEOUT'] = os.environ['D3MTIMEOUT']
+        env_dict['D3MCPU'] = (int)(os.environ['D3MCPU'])
+
+        with open(filepath, "w") as fp:
+            json.dump(env_dict, fp)
+
     # Garbage collect any leftover intermediate variables
     garbage_collect()
 
@@ -69,7 +116,7 @@ def fit_and_evaluate(output_dir):
         for idx, jsonfile in enumerate(os.listdir(pipeline_dir_path)):
             jsonfile_path = os.path.join(pipeline_dir_path, jsonfile)
             prediction_name = jsonfile.replace('.json', '')
-            print(f'Evaluating over JSON File {idx+1}/{len(os.listdir(pipeline_dir_path))}: {jsonfile_path}\n')
+            logging.critical(f'Evaluating over JSON File {idx+1}/{len(os.listdir(pipeline_dir_path))}: {jsonfile_path}\n')
 
             p = subprocess.Popen([sys.executable, '-m', 'd3m', 'runtime', '-v',
                                   os.environ['D3MSTATICDIR'], 'fit-produce', '-p',
@@ -87,13 +134,9 @@ def fit_and_evaluate(output_dir):
             # Generate code that converts JSON to python code
             src_dir = os.path.dirname(os.path.abspath(__file__))
 
-            pipeline_proc = subprocess.Popen([sys.executable, os.path.join(src_dir, 'mkpline.py'), 
-                                                jsonfile_path, os.path.join(search_dir_path, 'executables', f'{prediction_name}.code.py')],
-                                                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-
-            _, error = pipeline_proc.communicate()
-            if p.returncode != 0:
-                raise RuntimeError(error.decode())
+            subprocess.run([sys.executable, os.path.join(src_dir, 'mkpline.py'), 
+                                jsonfile_path, os.path.join(search_dir_path, 'executables', f'{prediction_name}.code.py')],
+                                stdout=sys.stdout, stderr=sys.stdout, check=True)
 
 
 def main_search():
@@ -110,7 +153,11 @@ def main_search():
     os.environ['D3MTIMEOUT'] = sys.argv[3]
     os.environ['D3MCPU'] = sys.argv[4]
     os.environ['D3MPROBLEMPATH'] = sys.argv[5]
-    os.environ['D3MSTATICDIR'] = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'static')
+
+    if len(sys.argv) == 7:
+        os.environ['D3MSTATICDIR'] = sys.argv[6]
+    else:
+        os.environ['D3MSTATICDIR'] = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'static')
 
     search.search_phase()
 
@@ -136,4 +183,5 @@ def main(argv):
             server.stop(0)
 
 if __name__ == '__main__':
-    main(sys.argv[1:])
+    #main(sys.argv[1:])
+    main_run()
