@@ -635,7 +635,8 @@ class SolutionDescription(object):
         primitive_hyperparams = primitive.metadata.query()['primitive_code']['class_type_arguments']['Hyperparams']
         method_arguments = primitive.metadata.query()['primitive_code'].get('instance_methods', {}).get('__init__', {}).get('arguments', [])
         if 'volumes' in method_arguments:
-            volumes = helper_functions.get_primitive_volumes(self.volumes_dir, primitive)
+            primitive_name = str(primitive)
+            volumes = helper_functions.get_primitive_volumes(self.volumes_dir, primitive, primitive_name)
             model = primitive(volumes=volumes, hyperparams=primitive_hyperparams(primitive_hyperparams.defaults(), **custom_hyperparams))
         else:
             model = primitive(hyperparams=primitive_hyperparams(primitive_hyperparams.defaults(), **custom_hyperparams))
